<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/class/dao/Maintain.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | Author: Author Name <EMAIL>                                            |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// $Id: Maintain.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * Maintain.php - contains the {@link DAO_Maintain} class
  *
  * @author $Author: morgabra $
  * @filesource
  *
  * @package    classes
  * @subpackage dao
  */

require_once("DB/DataObject.php");

/**
  * Base class from which all other DB_DataObjects in Maintain are extended
  *
  * @abstract
  * @package    classes
  * @subpackage dao
  */
class DAO_Maintain extends DB_DataObject
{

    // {{{ insert()

    /**
      * Some things all "insert"s need to do:
      *
      * 1) validate before insertion
      * 2) re-set all internal objects based on new values
      *
      * extends DB_DataObject's insert() method
      * @return mixed   key on success, or false on failure
      */
    function insert() {
        global $conf;

        // primary key
        if ($key = parent::insert()) {

            // re-setup all necessary objects
            $this->setAll();

        }

        return $key;
    }
    // }}}
    // {{{ update()

    /**
      * Things all updates need to do:
      *
      * 1) Validate before updating
      * 2) Set error messages
      * 3) Rebuild all internal objects
      *
      * @param  DB_DataObject   $orig   original DBDO for comparison
      * @return mixed   Number of rows updated (int) or false on error
      */
    function update($orig = false) {
        global $conf;

        $rows = parent::update($orig);

        // (re)build objects
        if ($rows === 1) {
            $this->setAll();
        }

        return $rows;
    }
    // }}}
    // {{{ validate()

    /**
      * Validate all database-standard variables
      *
      * @return mixed   true on success, array of results on failure
      */
    function validate() {

        // call DBDO's validate
        $result = parent::validate();

        // validate any other necessary variables
        $local = $this->_validate();

        // see if _validate() was all successful or not
        $eval = true;
        foreach ($local as $val) {
            if ($val === false) {
                $eval = false;
                break;
            }
        }

        // if the parent had errors, append local results
        if (is_array($result))
            return array_merge($result, $local);

        // if we had errors and the parent didn't, return ours
        elseif (!$eval)
            return $local;

        // otherwise, we made it
        return true;

    }

    // }}}
    // {{{ fetch()

    /**
      * Need to make sure we set all internals after a fetch
      * (extends DBDO's fetch() method)
      *
      * @return boolean true on success, false on failure
      */
    function fetch() {

        // now fetch the results
        $result = parent::fetch();

        if (!$result)
            return false;

        // set the relevant internals
        $this->setAll();

        return 1;
    }

    // }}}
    // {{{ toArray()

    /**
      * This is being overloaded to append the results of
      * ->_toArray(), so that classes with additional variables
      * can add more information to the resulting array, rather
      * than just those columns that appear in the database
      */
    function toArray()
    {
        // get the standard toArray() results
        $arr = parent::toArray();

        // append the results of the local ->_toArray() function
        $arr = array_merge($arr, $this->_toArray());

        // return the result
        return $arr;
    }

    // }}}

    // {{{ get_history()
    
    /**
      * Retrieve an Object_History object
      * for this object
      *
      * @param  boolean specify if data should be pulled from DB
      * @return mixed   DBDO object_history object, or false
      */
    function get_history($fetch = false) 
    {
        global $conf;

        // if no ID is set, we can't look up the history anyway
        if (!isset($this->id)) {
            return false;
        }

        $history = DB_DataObject::Factory('object_history');
        $history->object_type   = $this->_type();
        $history->object_id     = $this->id;

        if ($fetch) {
            $history->orderBy('date DESC');
            $history->find(true);
        }

        return $history;

    }

    // }}}
    // {{{ write_history()

    /**
      * Writes an object_history entry for this class
      * 
      * @param  string  comment to store
      * @param  string  action performed on this item (i.e. DELETE, UPDATE, CREATE)
      * @param  string  name of user who modified this object (defaults to Unknown User)
      * @param  int     ID of the parent zone of this object
      * @param  boolean true if history should be written even if not specified
      *                 for this object type in the config
      * @return mixed   true on success, PEAR_Error on error
      */
    function write_history($comment, $action, $username = '', $zone = 0, $force = false)
    {
        global $conf;

        $allowed = explode(',', $conf['history']['allow']);
        $type = $this->_type();

        // if we're not storing history information about this type, go away
        if (!$force && ($type == 'object_history' || !in_array($type, $allowed))) {
            return true;
        }

        // grab a blank history item for this object
        $history = $this->get_history();

        if (!$history) {
            return new PEAR_Error(_("Could not get history for this object type"));
        }

        // set the default username if none provided
        if (!$username) {
            $username = _("unknown user");
        }

        // try and figure out the zone if none provided
        if (!$zone) {
            $zone = ($this->zone) ? $this->zone : null;
        }

        // populate the object
        $history->username  = $username;
        $history->date      = time();
        $history->comment   = $comment;
        $history->action    = $action;
        $history->zone      = $zone;
        $history->summary   = $this->_history_text();

        $result = $history->insert();

        if (!$result) {
            return new PEAR_Error(_("Could not insert history for this object type"));
        }

        return true;
    }

    // }}}

/* "abstract" functions */
    // {{{ setAll()

    /**
      * When internal variables change, objects based
      * on those variables need updating as well.
      * Sometimes things like 'hostname' need to be 
      * re-evaluated if the domain or name changes
      * This function is used to set all of these
      * dependant internal variables and objects
      */
    function setAll() {}
    // }}}
    // {{{ can_edit()
    
    /**
      * Allows each object to determine who has editing rights
      *
      * @param  mixed   DAO_User object or username
      * @param  int     Zone ID to check permissions for
      * @return boolean false if user cannot edit, or rights cannot be determined
      */
    function can_edit($usr, $zone_id = false)
    {
        // make sure there's a user object
        if (!is_object($usr)) {
            $usr = new User($usr);
        }

        // use the user's current zone
        if (!$zone_id) {
            $zone_id = $usr->current_zone;
            if (!$zone_id) {
                return false;
            }
        }

        // by default, only ZONE_ADMIN has editing rights
        return $usr->is_zone_admin($zone_id);
    }

    // }}}
    // {{{ can_read()

    /**
      * Allows each object to determine who has read rights
      *
      * @param  mixed   DAO_User object or username
      * @param  int     ID of zone to check permissions for
      * @return boolean false if user cannot see this object
      */
    function can_read($usr, $zone_id = false)
    {
        // make sure there's a user object
        if (!is_object($usr)) {
            $usr = new User($usr);
        }

        // use the user's current zone
        if (!$zone_id) {
            $zone_id = $usr->current_zone;
            if (!$zone_id) {
                return false;
            }
        }

        // by default, anyone with ZONE_GUEST or greater has read permissions
        return $usr->is_zone_guest($zone_id);
    }

    // }}}
    // {{{ _toArray()

    /**
      * Internal method for chidl classes to add non-DB fields to their output
      *
      * This class should be used to call ->get*() for non-DB vars
      *
      * @access private
      * @return array   'varname'=>'value' array for display purposes
      */
    function _toArray()
    {
        return array();
    }

    // }}}
    // {{{ _validate()

    /**
      * Validate() only validates db-specific variables
      * This internal function _validate() allows each subclass
      * to validate any other internal variables they are storing
      *
      * This method should be used to call ->validate*() on non-DB vars
      *
      * @access private
      * @return array   array of 'varname'=>result
      */
    function _validate() {
        return array();
    }

    // }}}
    // {{{ _type()

    /**
      * For object_history, we need to know the 'type',
      * which is usually the class name, but if it's not
      * we want it to be extensible
      *
      * @access private
      * @return string  lower-case object_history index for this object type
      */
    function _type() 
    {

        // get the table represented by this class
        $class = strtolower($this->tableName());

        // this is unfortunate, but how it has to be for compatability:
        // strip the 'zone_' prefix off of the mappings, and hard-code
        // certain tables to ignore.  For example, cname should be
        // 'cname', but zone_user should NOT be 'user'
        if (substr($class, 0, 5) == 'zone_') {

            // get the stripped class
            $tmp_class = substr($class, 5);

            // there are certain zone_* tables we want to leave alone (the mappers)
            $ignore = array('range', 'workgroup', 'user');

            // use the stripped classname if it's not being ignored
            if (!in_array($tmp_class, $ignore)) {
                $class = $tmp_class;
            }

        } // if zone_

        return $class;
    }
    
    // }}}
    // {{{ _history_text()

    /**
      * A textual representation of an object to be stored in object_history's
      * summary column
      *
      * @access private
      * @return string  message text to append
      */
    function _history_text()
    {
        return serialize($this);
    }

    // }}}
    
}

?>
