<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/class/dao/extended/ip_blocks.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: ip_blocks.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * ip_blocks.php - contains the {@link IPBlock} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

require_once DAODIR.'/Ip_blocks.php';

/**
  * An IP Block
  *
  * @package    classes
  * @subpackage dao
  */
class IPBlock extends DAO_Ip_blocks {

    // {{{ constructor

    /**
      * Constructor
      *
      * @param  int ID (primary key) of IP Block or none for new block
      */
    function IPBlock ($id = 0)
    {
        if ($id) {
            $this->id = $id;
            $this->get($this->id);
        }
    
    }

    // }}}

/* Setters */
    // {{{ setEnabled

    /**
      * Set the 'enabled' flag for this IP_block
      *
      * @param  mixed   false/'false'/0 for false, everything else true
      */
    function setEnabled($en)
    {
        if ($en===false || $en==='false' || $en===0) {
            $this->enabled = 0;
        } else {
            $this->enabled = 1;
        }

        return true;
    }

    // }}}
    // {{{ setStart()

    /**
      * Set the starting address for this block
      *
      * @param  mixed   IP address (integer or dotted quad)
      */
    function setStart($start)
    {
        $ip = new IP($start);
        $this->startorigin = $ip->origin;
        $this->start = $ip->ip_int;
    }

    // }}}
    // {{{ setEnd()

    /**
      * Set the ending address for this block
      *
      * @param  mixed   IP address (integer or dotted quad)
      */
    function setEnd($end)
    {
        $ip = new IP($end);
        $this->endorigin = $ip->origin;
        $this->end = $ip->ip_int;
    }

    // }}}

/* Internal Functions */
    // {{{ find_ip_block()

    /**
      * Check to see if IP Block Exists
      *
      * @param  mixed   Start IP address (int or dottted quad)
      * @param  mixed   End IP address (int or dotted quad)
      * @param  string  name of IP block
      * @return mixed   ID (primary key) if block exists, false otherwise
      */
	function find_ip_block($start,$end,$name) 
    {
		global $conf;

		// Considers it a duplicate if name the same or if the ips are already in the db

        // "SELECT id FROM ip_blocks WHERE ( start<='$start' AND end>='$end' ) OR name='$name'"
        $dbdo = DB_DataObject::Factory('ip_blocks');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->whereAdd("( start<='$start' AND end>='$end') ");
        $dbdo->whereAdd("name='$name'", 'OR');

        if ($dbdo->find(true)) {
            return $dbdo->id;
        }
        
        return false;

	} //check_ip_block

    // }}}

/* Getters */
    // {{{ getStart()
    function getStart()
    {
        return IP::int2ip($this->start);
    }
    // }}}
    // {{{ getEnd()
    function getEnd()
    {
        return IP::int2ip($this->end);
    }
    // }}}

/*Validation Methods*/
//{{{ validateStart()

    function validateStart()
    {
        if(!isset($this->start))
            return _("An IP Block must have a start address");
        if($this->start >= $this->end)
            return _("The start IP must be lower than the ending IP");
        
        
        if($this->startorigin == "other")
            return _("Entry is not a valid IP address; it must be a valid dotted quad (x.x.x.x).");

        // find out anything that overlaps with the start ip
        $dbdo = DB_DataObject::Factory('ip_blocks');

        //They start before we do, and end after we do.  They contain us
        $dbdo->whereAdd("(start<='{$this->start}' AND end >='{$this->end}'");

        //They start before we do, end after we start
        $dbdo->whereAdd("start<='{$this->start}' AND end >='{$this->start}'","OR");

        //They start after we do, and end before we do, we contain them
        $dbdo->whereAdd("start>='{$this->start}' AND end<='{$this->end}')","OR");

        if (isset($this->id))
            $dbdo->whereAdd("ID!='{$this->id}'");

        
        // display the errors if ranges are found
        if ($dbdo->find(true)) {
            $tmp = new IPBlock($dbdo->id);
            $info = IP::int2ip($tmp->start).' - '.IP::int2ip($tmp->end);

            $msg    = _("The specified starting IP overlaps with another block").
                " ($info)";
            return $msg;

        }
        return true;
    }

//}}}
//{{{ validateEnd()

    function validateEnd()
    {
        if(!isset($this->end))
            return _("An IP address must have an ending IP");
        if($this->end <= $this->start)
            return _("The ending IP address must be greater than the starting IP");
        
        if($this->endorigin == "other")
            return _("Entry is not a valid IP address; it must be a valid dotted quad (x.x.x.x).");
            
        // find anything that overlaps with the ending ip 
        $dbdo = DB_DataObject::Factory('ip_blocks');

        //They start before we do and end after we do.  They contain us.
        $dbdo->whereAdd("(start<='{$this->start}' AND end >='{$this->end}'");
        
        //They start after we do, but before we end
        $dbdo->whereAdd("start>='{$this->start}' AND start<='{$this->end}'","OR");

        //Start after is. and end before us.  We contain them.
        $dbdo->whereAdd("start>='{$this->start}' AND end<='{$this->end}')","OR");
        if (isset($this->id))
            $dbdo->whereAdd("ID!='{$this->id}'");

        // display the errors if ranges are found
        if ($dbdo->find(true)) {
            $tmp = new IPBlock($dbdo->id);
            $info = IP::int2ip($tmp->start).' - '.IP::int2ip($tmp->end);

            $msg    = _("The specified ending IP overlaps with another block").
                " ($info)";
            return $msg;

        }

        return true;
    }

//}}}

/* Maintain Overloads */
    // {{{ _history_text()

    function _history_text()
    {
        // : NAME (xxx.xxx.xxx.xxx - xxx.xxx.xxx.xxx)
        return $this->getName() . '( ' . $this->getStart() . ' - ' .
                $this->getEnd() . ' )';
    }
    // }}}

}

?>
