<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/class/dao/extended/range.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: range.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * range.php - contains the {@link Range} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

/*
 * Oh boy: mostly extends DAO_Ranges, but has some
 * pieces of DAO_Zone_range in it :-(
 */

include_once DAODIR.'/Ranges.php';

/**
  * A range of IPs assigned to a @see Subnet
  *
  * @package    classes
  * @subpackage dao
  */
class Range extends DAO_Ranges {

    // {{{ constructor
    // we can populate this stuff from the database -- no sense in
    //   passing it in.
    function Range ( $range_id = 0 ) 
    {
        if ($range_id) {
            $this->id = $range_id;
            $this->get($this->id);
        }
        /* parent was getting left as null */
        $this->parent = 0;
    }
    // }}}

// DataObject overloads
    // {{{ delete()
    //Delete this range
    function delete($where = false)
    {

        /*Delete associated objects*/
        //Get the and host table
        $hosts = DB_DataObject::Factory('host');
        //Check if the range is static or dynamic
        if($this->type == "static") {  //If static, delete every host between the start and ending IP
        $hosts->whereAdd("ip >= '$this->start'");
        $hosts->whereAdd("ip <= '$this->end'");
        } else { //Otherwise, (it's dynamic) delete every host with the matching dynamic_range id
        $hosts->whereAdd("dynamic_range = $this->id");
        }
        $hosts->delete(DB_DATAOBJECT_WHEREADD_ONLY); //send delete query
        
        // Delete all the zone_range references
        $dbdo = DB_DataObject::Factory('zone_range');
        $dbdo->whereAdd("range='$this->id'");
        $dbdo->delete(DB_DATAOBJECT_WHEREADD_ONLY);
        
        // Delete the range itself
        $result = parent::delete($where);

        // just in case we're still using this object, we'll null the ID
        unset($this->id);

        return $result;

    } //delete
    // }}}
    // {{{ update()

    /**
      * If the type has changed, there are some actions that need to be taken
      */
    /* If we are splitting the range we don't need to check for orphaned
       hosts or type changes */
    function update($original = false)
    {

        // we need to compare the current type to the type in the database
        $other = DB_DataObject::Factory('ranges');
        $other->get($this->id);

        // if type has changed, convert it before updating
        if ($other->type != $this->type) {
            // convert this range to the new type
            $success = $this->_convert();

            // if it didn't convert, don't do the update
            if (!$success) {
                return $success;
            }

        } // if new type

        // similarly, if the end has shrunk, 
        // make sure there aren't orphaned hosts
        if ($this->end < $other->end) {

            //DB_DataObject::debugLevel(1);
            // see if any hosts have IPs in this now-nonexistant IP space
            $dbdo = DB_DataObject::Factory('host');
            $dbdo->whereAdd("ip>'$this->end'");
            $dbdo->whereAdd("ip<='$other->end'");

            /* if there were any see if there is another range that has this ip (this is for splitting ranges) */ 
            
            if ($dbdo->find()) {
                $result = false;
                /* Get the first host's ip */
                while($dbdo->fetch())   {
                    /* We don't want the string rep we want the int so don't use getIp */
                    $host_ip = $dbdo->ip;
                    /* Find a range with this IP else fail validation (don't leave host orphaned */
                    $cover_range = DB_DataObject::Factory('ranges');
                    $cover_range->whereAdd("start<='$host_ip'");
                    $cover_range->whereAdd("end>='$host_ip'");
                    if ($cover_range->find())   {
                        /* The ip is in another range */
                        $result = true;
                    }
                    else    { 
                        print "false on line 176: host it = $host_ip";
                        /* If there is a host ip w/o a range fail */
                        return false;
                    }
                }
            }

        } // if orphaned hosts

        // update this range
        return  parent::update($original);
    }

    // }}}

// Setters
    // {{{ setAll()
    function setAll() 
    {
        $this->setStart($this->start);
        $this->setEnd($this->end);
        $this->setSubnet($this->subnet);
    } //setAll
    // }}}
    // {{{ setStart()
    function setStart($ip) 
    {
        // ok, we need to make sure this is in standard form
        $tmp = new IP($ip);
        $this->startOrigin = $tmp->origin;
        $this->start = $tmp->ip_int;

        return true;
    } //setStart
    // }}}
    // {{{ setEnd()
    function setEnd($ip) 
    {
        // standardize the input
        $tmp = new IP($ip);
        $this->endOrigin = $tmp->origin;
        $this->end = $tmp->ip_int;

        return true;
    } // setEnd
    // }}}
    // {{{ setEnabled()

    /**
      * @param  mixed   true/false, 1/0, 'true'/'false'
      * @return bool    true on success
      */
    function setEnabled($enabled) 
    {
        if ($enabled === 'false' || $enabled === false || $enabled == 0) {
            $this->enabled = 0;
        } else {
            $this->enabled = 1;
        }

        return true;
    }
    // }}}
    // {{{ setAllow_all_hosts()

    /**
      * @param  mixed   true/false, 'true'/'false', 0/1
      * @return bool    true on success
      */
    function setAllow_all_hosts($allow = '') 
    {
        if ($allow === 'true' || $allow === true || $allow == 1) {
            $this->allow_all_hosts = 1;
        } else {
            $this->allow_all_hosts = 0;
        }

        return true;

    }
    // }}}
    // {{{ setType()
    function setType($type) 
    {

        if (!$type) {
            $this->type = 'static';
        } else {
            $this->type = $type;
        }

        return true;
    }
    // }}}
    // {{{ setSubnet()

    /**
      * Set the subnet for this range
      *
      * @param  int subnet ID (primary key)
      * @return boolean true on success
      */
    function setSubnet($subnet)
    {
        if (!$subnet) {

            // if no subnet was specified
            // determine the proper subnet by the IP values of this range
            if ($this->start) {
                $this->subnet = $this->_get_subnet($this->start);
            } elseif ($this->end) {
                $this->subnet = $this->_get_subnet($this->end);
            } else {
                return false;
            }
        } else {
            $this->subnet = $subnet;
        }
    }

    // }}}

// Getters
    // {{{ getStart()
    function getStart()
    {
        return IP::int2ip($this->start);
    }
    // }}}
    // {{{ getEnd()
    function getEnd()
    {
        return IP::int2ip($this->end);
    }
    // }}}
    // {{{ getSubnet()
    function getSubnet()
    {
        $subnet = DB_DataObject::Factory('subnet');
        $subnet->get($this->subnet);
        return "$subnet->name (".IP::int2ip($subnet->subnet).")";
    }
    // }}}
    //{{{ getDisplay()
        /**
         * Returns a string that can be used to display this range
         * on a form or whatnot.
         *
         *  
         */
    function getDisplay()
    {
        return $this->getStart()." -- ".$this->getEnd();
        
    }
    //}}}
    //{{{ getDisplayDelete()
    function getDisplayDelete()
    {
        return $this->getDisplay()." and all child hosts in that range.";
    }
    //}}}

// validation
    // {{{ validateSubnet()
    function validateSubnet() {

        if(!isset($this->subnet))
            return _("No subnet has been specified");
        if($this->_get_subnet($this->start) != $this->_get_subnet($this->end))
            return _("Start and end IPs belong to different subnets");
        if($this->_get_subnet($this->start) != $this->subnet)
            return _("Subnet for specified IPs is different than object value");

        return true;
    }
    // }}}
    // {{{ validateStart()
    function validateStart() {
        global $conf;

        $res = true;

        // make sure one is set
        if (!isset($this->start))
            return _("No starting IP was defined");

        if ($this->startOrigin == "other")
            return _("The starting IP must be a valid IP in dotted quad format (x.x.x.x)");

        // start can't be more than end
        if ( $this->start > $this->end )
            return _("The IP start has to be less than the IP end");

        // find out anything that overlaps with the start ip
        $dbdo = DB_DataObject::Factory('ranges');
        $dbdo->whereAdd("(start<='{$this->start}' and end >='{$this->end}'");
        $dbdo->whereAdd("start<='{$this->start}' and end>='{$this->start}'","OR");
        $dbdo->whereAdd("start>='{$this->start}' and end<='{$this->end}')","OR");
        if (isset($this->id))
            $dbdo->whereAdd("id!='{$this->id}'");

        // display the errors if ranges are found
        if ($dbdo->find(true)) {
            $tmp_range = new Range($dbdo->id);
            $info = IP::int2ip($tmp_range->start).' - '.IP::int2ip($tmp_range->end);

            $msg    = _("The specified starting IP overlaps with another range").
                " ($info)";
            return $msg;

        }
        
        // make sure there is a containing IP block
        $dbdo = DB_DataObject::Factory('ip_blocks');
        $dbdo->whereAdd("start<='$this->start'");
        $dbdo->whereAdd("end>='$this->start'");

        if (!$dbdo->find())
            return _("Start IP is not contained within a defined IP block");
            
        if(!$this->_get_subnet($this->start)) {
            return _("Specified start IP does not fall within a defined subnet"); 
        }

        return true;

    }
    // }}}
    // {{{ validateEnd()
    function validateEnd() 
    {
        global $conf;

        // make sure one is set
        if (!isset($this->end))
            return _("No ending IP was defined");

        if ($this->endOrigin == "other")
            return _("The ending IP must be a valid IP in dotted quad format (x.x.x.x)");
        

        // make sure the start is less than the end, and that it's available
        // for this install of Maintain (the IPs exist in the ip table)
        if ($this->start >= $this->end)
            return _("The ending IP must be greater than the starting IP");

        // find anything that overlaps with the ending ip 
        $dbdo = DB_DataObject::Factory('ranges');
        $dbdo->whereAdd("(start<='{$this->start}' and end >='{$this->end}'");
        $dbdo->whereAdd("start>='{$this->start}' and start <='{$this->end}'","OR");
        $dbdo->whereAdd("start>='{$this->start}' and end<='{$this->end}')","OR");
        if (isset($this->id))
            $dbdo->whereAdd("id!='{$this->id}'");

        // display the errors if ranges are found
        if ($dbdo->find(true)) {
            $tmp_range = new Range($dbdo->id);
            $info = IP::int2ip($tmp_range->start).' - '.IP::int2ip($tmp_range->end);

            $msg    = _("The specified ending IP overlaps with another range").
                " ($info)";
            return $msg;

        }
        // make sure there is a containing IP block
        $dbdo = DB_DataObject::Factory('ip_blocks');
        $dbdo->whereAdd("start<='$this->end'");
        $dbdo->whereAdd("end>='$this->end'");

        if (!$dbdo->find())
            return _("End IP is not contained within a defined IP block");

        if(!$this->_get_subnet($this->end))
            return _("Specified end IP does not fall within a defined subnet");

        return true;
   
    }
    // }}}
//{{{   validateAllow_all_hosts()

/**
 *  Make sure we're allowed to allow all hosts
 *
 *  @return Mixed   True if valid, Error string otherwise.
 */
function validateAllow_all_hosts()
{
    if(strcasecmp($this->type, 'dynamic') && $this->allow_all_hosts == 1)
        return _('Range type must be dynamic to allow all dynamically registered hosts');
    return true;
}

//}}}

// Internal Functions
    // {{{ get_sibling_ranges()

    /**
      * Return a list of all ranges within the same subnet
      *
      * @return array   list of range IDs (primary key) in the same subnet
      */
    function get_sibling_ranges() 
    {
        $dbdo = DB_DataObject::Factory('ranges');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->subnet = $this->subnet;

        // ignore the current range
        $dbdo->whereAdd("id!='$this->id'");

        $dbdo->find();
        while ($dbdo->fetch())
            $results[] = $dbdo->id;
        
        if (!$results) return array();
        return $results;

    } //get_sibling_ranges

    // }}}
    // {{{ get_represented_domains()

    /**
      * Get a list of all domains present in this range
      *
      * @return array   domain IDs (primary keys) for all domains used by 
      *                 hosts in this range
      */
    function get_represented_domains() 
    {
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd("DISTINCT(domain) as domain");

        if ($this->type == 'static') {
            // "SELECT DISTINCT(host.domain) FROM host WHERE host.ip>='".$this->start_obj->ip_int."' AND host.ip<='".$this->end_obj->ip_int."'"
            $dbdo->whereAdd("ip>='$this->start'");
            $dbdo->whereAdd("ip<='$this->end'");
        }
        else {
            // "SELECT DISTINCT(host.domain) FROM host WHERE dynamic_range='$this->id'"
            $dbdo->dynamic_range = $this->id;
        }

        $dbdo->find();
        while ($dbdo->fetch())
            $results[] = $dbdo->domain;

        return $results;

    } // get_represented_domains

    // }}}
    // {{{ count_range_hosts()
    //get the hosts that are in this range....
    function count_range_hosts() 
    {
        global $conf;

        $dbdo = &$this->_prep_range_hosts();

        if (PEAR::isError($dbdo))
            return $dbdo;

        return $dbdo->find();

    } //count_range_hosts
    // }}}
    // {{{ get_range_hosts()

    /**
      * Return a list of hosts in this range
      *
      * @return array   list of host IDs (primary key) for hosts in this range
      */
    function get_range_hosts() 
    {
        global $conf;

        $dbdo = &$this->_prep_range_hosts();

        if (PEAR::isError($dbdo))
            return $dbdo;

        $dbdo->find();
        $hosts = array();
        while ($dbdo->fetch())
            $hosts[] = $dbdo->id;

        return $hosts;
    
    }

    // }}}
    // {{{ count_free_addresses()

    /**
      * Count the number of free addresses in this range
      *
      * @return int total addresses in range minus addresses in use
      *             (this can be negative for an overfilled dynamic range)
      *             returns PEAR_Error for unhandled range types
      */
    function count_free_addresses()
    {
        global $conf;

        // total IPs in this range
        $total = $this->end - $this->start + 1;

        // get the DB_DataObject for the query
        $dbdo = $this->_prep_free_ips();

        // propogate errors
        if (PEAR::isError($dbdo)) {
            return $dbdo;
        }

        // get the result
        $used = $dbdo->find();

        // return the difference times the overflow
        if ($this->type == "dynamic")
            return (($total * $conf['general']['dynamic_overflow_multiple']) - $used);
        
        return ($total - $used);

    }

    // }}}
    // {{{ get_free_addresses()

    /**
      * Return a list of free IP addresses in this range
      *
      * @param  int     limit the number of results
      * @return array   array of free IP addresses (as ints) in this range
      *                 or empty list for non-static ranges
      */
    function get_free_addresses($limit=0)
    {
        // as promised, get out of here if this isn't a static range
        if ($this->type != 'static') {
            return array();
        }
        // get all used IPs in ascending order
        $dbdo = $this->_prep_free_ips();
        $dbdo->orderBy('ip');
        $dbdo->find();

        // make used ip list
        $used = array();
        while ($dbdo->fetch()) {
            $used[] = $dbdo->ip;
        }

        // make a list of all IPs in the range
        $ips = array();
        for ($i=$this->start; $i<=$this->end; $i++) {
            $ips[] = $i;
        }

        // generate array difference between available and used IPs
        $free = array_diff($ips, $used);
        
        return $free;
    }

    // }}}
    // {{{ get_mergeable_ranges()

    /**
      * Find out what ranges can be merged with this one
      *
      * @return array   a list of range IDs that can be merged with this one
      */
    function get_mergeable_ranges()
    {
        global $conf;

        // start a list of results
        $ranges = array();

        // get a list of sibling ranges
        $others = $this->get_sibling_ranges();
        foreach ($others as $range) {
            $range2 = new Range($range);
            if ($this->_can_merge($range2)) {
                $ranges[] = $range;
            }
        }

        return $ranges;

    } //get_mergeable_ranges

    // }}}
//{{{ get_used_addresses()

    /**
     *  Return a list of used IPs in the range
     *
     *  @return Array   array of used IP addresses (as ints) in the range
     *                  or an empty array if the range is empty or dynamic.
     */
    function get_used_addresses()
    {
        $ips = array();
        if(!isset($this->type))
            return $ips;
        if(!strcasecmp($this->type,"dynamic"))
            return $ips;
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->whereAdd();
        $dbdo->whereAdd("ip >= '{$this->start}' AND ip <= '{$this->end}'");
        $dbdo->find();

        while($dbdo->fetch())
        {
            $ips[] = $dbdo->ip;
        }

        return $ips;
    }

//}}}

// Static Methods
    // {{{ merge_ranges()
    
    /**
      * Combine two ranges
      *
      * @param  int ID (primary key) for one range
      * @param  int ID (primary key) for the second range
      * @return int ID of new range (PEAR_Error on error)
      */
    function merge_ranges($first, $second)
    {
        /* order of operations:
         *
         * 1) append/prepend
         * 2) move all associated objects
         * 3) update the zone_range maps
         */

        // set up the range objects
        $range1 = new Range($first);
        $range2 = new Range($second);

        /* 1) Append one range to the other */

        // for scope: determine if merging was successful
        $success = false;

        // append the proper range
        if ($range1->end == ($range2->start - 1)) {
            $success = $range1->_append($range2);
        } elseif (($range1->start - 1) == $range2->end) {
            $success = $range2->_append($range1);
        } else {
            // the ranges aren't adjacent, get out with error
            return new PEAR_Error(
                    _("Ranges cannot be merged as they are not adjacent"));
        }

        // if the appending didn't work, break out of here
        if (PEAR::isError($success)) {
            return $success;
        }

        /* for steps 2) and 3) we need to know which was 
           the old range and which is new */
        $old = ($success == $first) ? $second : $first;

        /* 2) move all associated objects (i.e. hosts) */
        // only needed if dynamic range

        if ($this->type == 'dynamic') {

            // all hosts referring to the appended range should now refer to this one
            $host = DB_DataObject::Factory('host');
            $host->dynamic_range = $success;
            $host->whereAdd("dynamic_range='$old'");
            $host->update(DB_DATAOBJECT_WHEREADD_ONLY);

        }

        /* 3) Update the zone_range map */

        // all instances of the old range in zone_range change to this range
        $range = DB_DataObject::Factory('zone_range');
        $range->range = $success;
        $range->whereAdd("range='$old'");
        $range->update(DB_DATAOBJECT_WHEREADD_ONLY);
    }

    // }}}

//{{{  split_range()
    /*
     *  This function will split a range into two. Validation is 
     *  not done here and should be done in the form. 
     *
     *
     *  @param int ID (primary key) of the range to be split
     *  @param int new start ip (in int form) for the new range
     *  @return true on success or a PEAR_Error
     */

    function split_range($range_id, $range2_start_ip)
    {
        /* Process */
        /* Get the range from db and copy it     */
        /* Set the start and end ips accordingly */
        /* Update old one and insert new one     */
        $r1 = new Range($range_id);
        $r2 = $r1; //copy

        $r2->setStart($range2_start_ip);
        unset($r2->id);
        /* If this doesn't work we should go back and fix the original */
        $suc = $r2->insert();
        if (PEAR::isError($suc))   {
            return $suc;
        }
        $r1->setEnd($range2_start_ip-1);
       
        if ($r1->update() === false)    {
            $r2->delete();
            return new PEAR_Error(_("Problem updating database"));
        }
        return true;
    }
//}}}

// helper functions
    // {{{ _get_subnet()

    /**
      * Get the ID of the subnet this range belongs in
      *
      * This should be used for new ranges to determine the subnet value
      *
      * @access private
      * @param  mixed   IP (int or dotted quad) to use to find the subnet
      * @return mixed   subnet ID (primary key) or false if none found
      */
    function _get_subnet($ip_start=0) 
    {
        global $conf;
        
        //if we don't set use the objects
        if (!$ip_start) {
            $ip_start = $this->start;
        }

        //Get the correct subnet for this range

        // "SELECT id FROM subnet WHERE subnet<='$ip_start' ORDER BY subnet DESC LIMIT 1"
        $dbdo = DB_DataObject::Factory('subnet');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->whereAdd("subnet<='$ip_start'");
        $dbdo->orderBy('subnet DESC');

        if ($result = $dbdo->find(true)) {
            return $result;
        } else {
            return false;
        }
        
    } //_get_subnet

    // }}}
    // {{{ _prep_range_hosts()

    /**
      * get_range_hosts() and count_range_hosts() are the same up to the 
      * ->find() statement.  So, this handles all of the query setup, etc.,
      * and returns the prepped object
      *
      * @access private
      * @return DB_DataObject   prepared data object, or PEAR_Error
      */
    function _prep_range_hosts()
    {
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');

        if ($this->type == "static") {

            // "SELECT host.id FROM host WHERE ip>='$start' AND ip<='$end' ORDER BY ip DESC"
            $dbdo->whereAdd("ip>='$this->start'");
            $dbdo->whereAdd("ip<='$this->end'");
            $dbdo->orderBy('ip DESC');

        } elseif ($this->type == "dynamic") {
        
            // "SELECT host.id FROM host WHERE dynamic_range='$this->id'"
            $dbdo->SetDynamic_range($this->id);

        } else {
            return new PEAR_Error(_("Invalid type"));
        }

        return $dbdo;
    }

    // }}}
    // {{{ _prep_free_ips()

    /**
      * Prepare the DBDOs for the queries that determine the free IP addresses
      *
      * both count_free_addresses() and get_free_addresses() use this
      *
      * @access private
      * @return object  populated db_dataobject before ->find() is called
      */
    function _prep_free_ips()
    {
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();

        if ($this->type == 'static') {

            /* get a the number of IPs used by hosts */
            $dbdo->selectAdd("DISTINCT(ip) as ip");
            $dbdo->whereAdd("ip>='$this->start'");
            $dbdo->whereAdd("ip<='$this->end'");

        } elseif ($this->type == 'dynamic') {
            
            /* get the number of hosts in this range */
            $dbdo->selectAdd('dynamic_range');
            $dbdo->whereAdd("dynamic_range='$this->id'");

        } else {
            return new PEAR_Error(_("Cannot determine free IPs for range type:").
                    " $this->type");
        }

        return $dbdo;

    }

    // }}}
    // {{{ _append()

    /**
      * Append a range to this one
      *
      * @access private
      * @param  object  DAO_Range Object to be appended to thsi one
      * @return mixed   ID if new range on success, PEAR_Error on failure
      */
    function _append($range2)
    {
        // make sure the ranges can be combined:
        if (!$this->_can_merge($range2)) {
            return new PEAR_Error(_("Ranges cannot be combined."));
        }

        /* Do the appending */

        // set the new ending value to the argument's end
        $this->setEnd($range2->end);

        // if we couldn't do the update, throw an error
        if (!$this->update()) {
            return new PEAR_Error(_("Could not update range") . 
                    $this->_history_text());
        }

        /* Add an object_history entry indicating the merge */

        // remove the old range
        $range2->delete();

        // show the new range in history
        $msg = _("The range") . $range2->_history_text().' ';
        $msg .= _("has been merged with the range"). ' ' . $this->_history_text();

        // write the history
        $this->write_history($msg, ACTION_DELETE);

        return $this->id;
    }

    // }}}
    // {{{ _convert()

    /**
      * Switch this range from one type to another (i.e. static to dynamic)
      *
      * @access private
      * @return boolean true on success, false on failure
      */
    function _convert()
    {
        // either way, we're going to need a list of hosts for this range
        $hosts = $this->get_range_hosts();

        // if we're going from static to dynamic, convert the hosts
        if ($this->type == 'dynamic' || $this->type == 'ddns') {

            /* First things first, get all the hosts for this range */
            $hosts = $this->get_range_hosts();

            // update each host
            foreach($hosts as $host_id) {

                // instantiate the new host
                $host = new Host($host_id);

                // if it's not in this range, convert it
                $host->setDynamic_range($this->id);

                // if we updated it, add some object_history
                if ($host->update()) {
                    $msg = "$host->hostname " . _("converted to dynamic range")
                        . ' ' . $this->_history_text();

                    $host->write_history($msg, ACTION_MODIFY);
                }

            } //foreach host

            return true;

        } elseif ($this->type == 'static') {
            // if converting from dynamic to static, assign an IP to each host

            /* 1) Make sure the hosts can all fit into a static range */

            // get the number of free addresses
            $free = $this->count_free_addresses();

            // if they won't fit, exit false
            if ($free < 0) {
                return false;
            }

            // convert the hosts
            /* First things first, get all the hosts for this range */
            $hosts = $this->get_range_hosts();

            // for each host, find an open IP, assign it, write object history
            foreach($hosts as $host_id) {

                $host = new Host($host_id);
                $host->setIp($this->get_free_ip());

                // update and write the history
                if ($host->update()) {

                    $msg = "$host->hostname " . _("converted to static range") 
                        . ' ' . $this->_history_text() . ' ' . _("and given IP:")
                        . ' ' . IP::int2ip($host->getIp());

                    $host->write_history($msg, ACTION_MODIFY);
                }
            }

            return true;
        }

        // unknown range type...shouldn't happen anyway,
        // as we should be validating before this step
        return false;

    }

    // }}}
    // {{{ _can_merge()

    /**
      * Determine if specified range can be merged with this one
      *
      * Used by _append() and get_mergeable_ranges()
      *
      * @access private
      * @param  object  DAO_Ranges object to test
      * @return boolean true if subnet and type are the same and IPs are adjacent
      */
    function _can_merge($range2)
    {
        // make sure types are the same
        if ($range2->type != $this->type) {
            return false;
        }

        // make sure subnets are the same
        if ($range2->subnet != $this->subnet) {
            return false;
        }

        // make sure IPs are adjacent
        if (($range2->start - 1) != $this->end
            && ($range2->end + 1) != $this->start) {
            return false;
        }

        return true;

    }

    // }}}

    // Maintain Overloads
    // {{{ _history_text()
    function _history_text()
    {
        return $this->getStart().' - '.$this->getEnd()." $this->type";
    }
    // }}}

}

?>
