<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/class/dao/extended/subnet.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: subnet.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * subnet.php - contains the {@link Subnet} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

require_once DAODIR.'/Subnet.php';
require_once 'Net/IPv4.php';

/**
  * Subnet allocates part of an @see IPBlock
  *
  * @package    classes
  * @subpackage dao
  */
class Subnet extends DAO_Subnet
{

    // {{{ constructor

    function Subnet($subnet_id = 0)
    {
        global $conf;

        if ($subnet_id) {
            $this->id = $subnet_id;
            $this->get($this->id);
        }
    }

    // }}}

// DB_DataObject overloads
    // {{{ delete()

    function delete($usewhere = false)
    {
        global $user;

        $result = parent::delete($usewhere);

        if (!$result)
            return $result;

		// Delete any object_history
		// "DELETE FROM object_history WHERE type='subnet' AND object_id='$this->id'";

        $dbdo = DB_DataObject::Factory('object_history');
        $dbdo->whereAdd("type='subnet'");
        $dbdo->whereAdd("object_id='{$this->id}'");
        $dbdo->delete(true);

        return $result;

    }

    // }}}

// Built-In Functions
    // {{{ get_last_ip()

    /**
      * Return (as an int) the last IP in this subnet
      * based on the internal values of subnet/netmask
      *
      * @return int last IP in this subnet or PEAR_Error
      */
	function get_last_ip()
    {
		global $conf;

        // since a calculated broadcast address will be the same as 
        // the 'last address', we'll let Net_IPv4 handle this

        $ip = new Net_IPv4();
        $ip->netmask    = IP::int2ip($this->netmask);
        $ip->ip         = IP::int2ip($this->subnet);
        $result = $ip->calculate();

        if (PEAR::isError($result))
            return $result;
        
        return $ip->broadcast;
    }

    // }}}
    // {{{ get_options()

    /**
      * Get the local options for this subnet
      *
      * @return array   list of object_option ids for this subnet
      */
	function get_options() 
    {
		global $conf;

        // "SELECT id FROM object_option WHERE type='subnet' AND object_id=$this->id"
        $dbdo = DB_DataObject::Factory('object_option');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');

        $dbdo->type = 'subnet';
        $dbdo->object_id = $this->id;
        $dbdo->find();

		while ($dbdo->fetch()) {
            $options[] = $dbdo->id;
        }

		return $options;

	} // get_options

    // }}}

// Setters
    // {{{ setSubnet()
    function setSubnet($subnet)
    {
        $ip = new IP($subnet);
        $this->subnetOrigin = $ip->origin;
        $this->subnet = $ip->ip_int;
        return true;
    }
    // }}}
    // {{{ setNetmask()
    function setNetmask($nm)
    {
        $ip = new IP($nm);
        $this->netmaskOrigin = $ip->origin;
        $this->netmask = $ip->ip_int;
        return true;
    }
    // }}}

// Getters
    // {{{ getSubnet()
    function getSubnet()
    {
        return IP::int2ip($this->subnet);
    }
    // }}}
    // {{{ getNetmask()
    function getNetmask()
    {
        return IP::int2ip($this->netmask);
    }
    // }}}
    // {{{ getVlan()
    function getVlan()
    {
        $vlan = DB_DataObject::Factory('vlan');
        $vlan->get($this->vlan);
        return $vlan->name;
    }
    // }}}

// Validators
    // {{{ validateSubnet()

	function validateSubnet()
    {
		global $conf;

        // make sure the subnet is a valid IP
        if ($this->subnetOrigin == "other")
            return _("Given subnet is not a valid IP in dotted quad format (x.x.x.x)");

        // make sure the subnet IPs exist in the database
        if (!IP::ip_exists($this->subnet)) {
            return _("You have specified a subnet inside of an IP block that has not been created or enabled yet");
        }


        // we're going to need the last ip in this subnet
        // if obtaining that fails, we need to exit with an error
        $end = IP::ip2int($this->get_last_ip());
        if (PEAR::isError($end)) {
            return $end->getMessage();
        }

        // make sure the subnet IPs exist in the database
        if (!IP::ip_exists($end)) {
            return _("You have specified a subnet inside of an IP block that has not been created yet");
        }

		//check and see if there is a subnet the overlaps after this one

        // "SELECT id FROM subnet WHERE subnet >= $subnet_int AND subnet <= $subnet_end AND id!=$this->id"
        $dbdo = DB_DataObject::Factory('subnet');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->selectAdd('subnet');
        $dbdo->whereAdd("subnet >= '$this->subnet'");
        $dbdo->whereAdd("subnet <= '$end'");
        if ($this->id)
            $dbdo->whereAdd("id!='$this->id'");

        if ($dbdo->find(true)) {
            return _("This subnet overlaps with subnet: ") .
                " $dbdo->name (".IP::int2ip($dbdo->subnet).")";
        }

        return true;

	} //validateSubnet

    // }}}
    // {{{ validateNetmask()

    function validateNetmask()
    {
        global $conf;

        if ($this->netmaskOrigin == "other")
            return _("Given netmask is not a valid IP in dotted quad format (x.x.x.x).");

        if (!Net_IPv4::validateNetmask(IP::int2ip($this->netmask)))
            return _("Given IP is not a valid netmask.");

        return true;
    }

    // }}}

// Internal Functions
    // {{{ _history_text()

    function _history_text()
    {
        return $this->getSubnet() . _("mask:") . ' ' . $this->getNetmask();
    }

    // }}}

} // class Subnet

?>
