<?php 

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL:$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: DHCPOEditForm.php 3988 2005-12-14 20:10:29Z bartel $

/**
 *  DHCPOEditForm.php - This form contains the {@link DHCPOEditForm} class
 *
 *  @author     $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  DHCPOEditForm - The creation form for DHCP Options
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class DHCPOEditForm extends DHCPOCreateForm
{

//{{{   form_init_data()
    /**
     *  This method initializes the data in the edit form to the current data in the database... Hence the name.   
     */
    function form_init_data()
    {   
        $do = new DHCPOption($_REQUEST['id']);
        $do->query("BEGIN");
        $init = $this->remap($do->toArray(), FORM);
        foreach ($init as $var=>$val)
            $this->set_element_value($var, $val);
        $do->query("COMMIT");
    }

//}}}
//{{{   form_action()

    /**
     *  This method is performed after validation, and is responisble for actually
     *  modifying the option in the database.
     *
     *  @return Boolean True if successfully added to the database
     */
    function form_action()
    {
        global $user;

        $res = true;
        $dbdo = new DHCPOption($_REQUEST['id']);
        $dbdo->query("BEGIN");
        $vals = $this->get_all_element_values();
        
        if(!isset($vals['custom']))
            $vals['custom'] = 0;
            
        $objary = $this->remap($vals, DBDO);
        $dbdo->setFrom($objary);
        
        if(!$dbdo->update()) {
            $dbdo->query("ROLLBACK");
            $res = false;
            $this->add_error(_("ERROR"), _("Could not update the database"));
        } else {
            $dbdo->query("COMMIT");
            $dbdo->write_history(_("DHCP Option Edited"), ACTION_MODIFY,
                                $user->username, $_SESSION['zone']);
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method is ran if form_action() was successful, we use it to display
     *  a success message to the users upon completion of the edit.
     *
     *  @return DIVTag  $div    Div that contains a success message
     */
    function form_success()
    {
        /*
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."option.php", _("[Return to Home]"));
        $link = html_a(web_root()."index.php", _("[Return to DHCP Option List]"));
        $div->add(container(_("DHCP Option Edited Successfully"), html_br()));
        $div->add(container($indlink,$link));
        */
        /* Get outa here */
        //$this->set_action(_("success"));
        header("Location: ".$_SERVER['SCRIPT_NAME']);
        return $div;
    }

//}}}
//{{{   form_hidden()

    /**
     *  Add some hidden values that we'll need later.  The ID of the OPTION and the
     *  get action so we can get back to the form.
     *
     *  @return Container   Container that has the hidden elements in it
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));
    }

//}}}

}

?>
