<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: DeleteZoneUserConfirm.php 4822 2006-09-15 18:57:40Z morgabra $

/**
 *  DeleteZoneUserConfirm.php - Contains the {@link DeleteZoneUserConfirm}
 *                              class
 *  @author     $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

/**
 *  This class is the delete confirm page
 *
 *  @package    htdocs
 *  @subpackege forms
 */
class DeleteZoneUserConfirm extends FormContent
{

//{{{   form()
    /**
     *  This method lays the form out for display.
     *
     *  @return DIVTag  $div    Div that contains the form
     */
    function form()
    {
        $usr = DB_DataObject::Factory('zone_user');
        $usr->id = $_REQUEST['id'];
        $usr->find(1);
        
        $div = html_div("delconfirm");
        $cont = container();
        $cont->add(_("You are going to delete ").$usr->username." from zone ". $usr->zone);
        $cont->add(html_br());
        $cont->add(_("Are you sure?"));
        $cont->add(html_br(),$this->add_action(_("Confirm")),$this->add_action(_("Cancel")));
        $cont->add($this->element_form("action"));
        $cont->add($this->element_form("id"));
        $div->add($cont);

        return $div;
    }

//}}}
//{{{   form_action

    /**
     *  This method defines the behavior for the form based on the button pressed
     *
     *  @return boolean     true if action was successful
     */
    function form_action()
    {
        global $user;
        
        switch(strtolower($this->get_action()))
        {
            case 'confirm':
                $usr = DB_DataObject::Factory('zone_user');
                $usr->query("BEGIN");
                $usr->id = $_REQUEST['id'];
                
                $usr->find(1);
                
                $name = $usr->username;
                $zoneid = $usr->zone;

                // No, you can't delete yourself from a zone.
                // This would allow admins to lock out themselves.
                if ($user->username == $name) {
                    $usr->query("ROLLBACK");
                    $this->add_error(_("Suicide attempt"), _("A user can not delete himself from the zone. If this is necessary, please ask an admin to delete you from the desired zone."));
                    return false;
                }
                
                if($usr->delete() == true) {
                    $usr->query("COMMIT");
                    $usr->write_history(_("Deleted User:")." ".$name." ".
                                        _("from zone:")." ".$zoneid,
                                        ACTION_DELETE,$user->username, $zoneid);
                } else {
                    $usr->query("ROLLBACK");
                    return false;
                }
                break;
            case 'cancel':
                header("Location: user.php");
                break;
            default:
                die_hard(_("Invalid form action"));
                break;
        }
        return true;
    }
//}}}
//{{{   form_success()

    /**
     *  This method displays a message informing the user that the delete was
     *  successful.
     *
     *  @return DIVTag  $div    The div that contains the message
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a("index.php", "[Return to Home]");
        $hoslink = html_a("user.php", "[Return to User List]");
        $div->add(container("User deleted successfully!",html_br()));
        $div->add(container($indlink, $hoslink));
        return $div;
    }

//}}}
//{{{   form_init_elements()

    /**
     *  This method inits the elements that are able to be included in the form.
     */
    function form_init_elements()
    {
        $this->add_element(new FEHidden("action","delzoneuser"));
        $this->add_element(new FEHidden("id", $_REQEST['id']));
        $this->set_action($_SERVER["SCRIPT_NAME"]);
    }

//}}}

}

?>
