<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: FullAdminForm.php 4822 2006-09-15 18:57:40Z morgabra $

/**
 *  FullAdminForm.php - this file contains the {@link FullAdminForm} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class is the form for making an existing user a full admin
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class FullAdminForm extends MaintainForm
{

//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that are going to be in the form
     */
    function form_init_elements()
    {
        $username = new FEText(_("Username"),true);

        $this->add_element($username);
        $this->set_action($_SERVER["SCRIPT_NAME"]);
        $this->set_cancel_action(web_root().'admin.php');
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method adds the hidden items to the form
     */
    function form_hidden()
    {
        $act = new FEHidden("action","fulladmin");
        $this->add_element($act);
        return $this->element_form("action");
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method makes sure that the data is valid before it processed
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true;
        
        $dbdobj = DB_DataObject::Factory('user');
        $dbdobj->query("BEGIN");
        $dbdobj->username = $this->get_element_value(_("Username"));
        
        // If the user doesn't exist
        if(!$dbdobj->find()) {
            $res = false;
            $this->add_error(_("Username"), _("User does not exist."));
            return false;
        }
        $dbdobj->query("COMMIT");

        return true;
    }

//}}}
//{{{   form_action()

    /**
     *  This method processes the form values after validation has been confirmed
     *
     *  @return Boolean     True if action was completed successfully
     */
    function form_action()
    {
        global $user;
        $res = true;
    
        $uname = $this->get_element_value(_("Username"));
        
        //Find all existing instances of the user and delete them
        $dbdobj = DB_DataObject::Factory('zone_user');
        $dbdobj->query("BEGIN");
        $dbdobj->username = $uname;
        $dbdobj->find();
        while($dbdobj->fetch()) {
            $dbdobj->delete();
        }
        
        //Put them in their new home... zone 0
        $dbdobj = DB_DataObject::Factory("zone_user");
        $dbdobj->username = $uname;
        $dbdobj->zone = 0;
        $dbdobj->level = ADMIN;

        if(!$dbdobj->insert()) { //If this fails report an error
            $dbdobj->query("ROLLBACK");
            $res = false;
            $this->add_error(_("ERROR"), _("Failed to make"). ' ' . $uname . ' '. _("a full admin"));
        } else {
            $dbdobj->query("COMMIT");
            $dbdobj->write_history(_("Made full admin"),ACTION_UPDATE, $user->username);
        }

        return $res; 
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a message reporting to the user that the action was
     *  a success
     *
     *  @return DIVTag  $div    Div with the success message inside
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a("index.php", _("[Return to Home]"));
        $admlink = html_a(_("admin.php"), _("[Return to Admin Control]"));
        $div->add(container(_("User successfully made a full admin!"), html_br()));
        $div->add(container($indlink, $admlink));
        return $div;
        
    }

//}}}

}

?>
