<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: MXEditForm.php 4822 2006-09-15 18:57:40Z morgabra $

/**
 *  MXEditForm.php - contains the {@link MXEditForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MXCreateForm.php");

class MXEditForm extends MXCreateForm
{

//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that will be used.  Please refer
     *  to {@link MXCreateForm} for more information.
     */
    function form_init_elements()
    {
        parent::form_init_elements();
        $enable = new FEListBox(_("Status"));
        $comment = new FETextArea(_("Comment"), true, 5, 40);

        $enable->set_list_data(array("Enabled"=>1, "Disabled"=>0));
       
        $this->add_elements($enable, $comment);
    }

//}}}
//{{{   form_init_data()
   
    /**
     *  This method sets the initial values for the form elements based off of
     *  which MX record we're updating.
     */
    function form_init_data()
    {
        $mx = new MX($_REQUEST['id']);
        $formarray = $this->remap($mx->toArray(), FORM);

        foreach($formarray as $label=>$val)
            $this->set_element_value($label, $val);
        
        $this->set_element_value(_("Comment"), _("Modified MX"));
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method establishes the hidden elements for the form.
     *
     *  @return Container   Container with the hidden elements for the form
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));                                
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what actually handles the updating of the MXRecord using
     *  the form values.
     *
     *  @return Boolean True if action succeeds
     */
    function form_action()
    {
        global $user;
        
        $res = true;
        
        switch(strtolower($this->get_action())) {

        case 'submit':
            $mx = new MX($_REQUEST['id']);
            $mx->query("BEGIN");
            $ary = $this->get_all_element_values();
            $mx->setFrom($this->remap($ary, DBDO));

            if($ary[_('Entire_Domain')]) {
                $dom = new Domain($vals[_("Domain")]);
                $mx->setDomain($dom->master_domain);
                $mx->setName(strtok($dom->name,'.'));
            }
 

            if($mx->update() === false) {
                $mx->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), _("Failed to modify MX Record in Database"));
            } else {
                $mx->query("COMMIt");
                $mx->write_history($ary[_('Comment')], ACTION_MODIFY, $user->username, 
                                    $_SESSION['zone']);
            }
            break;

        default:
            die_hard(_("Invalid form action"));
            break;

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This displays a happy message to the user letting them know that the update
     *  worked.
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success() 
    {
        $div = html_div('msuccess');
        $div->add(_("Successfully Modified MX Record"), html_br());
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $sublink = html_a(web_root()."mx.php", _("[Return to MX List]"));

        $div->add($indlink, $sublink);

        return $div;
    }

//}}}

}

?>
