<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: PointerEditForm.php 4822 2006-09-15 18:57:40Z morgabra $

/**
 *  PointerEditForm.php - This file contains the {@link PointerEditForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/PointerCreateForm.php");

/**
 *  This class is the form used to edit pointer objects
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class PointerEditForm extends PointerCreateForm 
{

//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that are going to be used
     *  in the form.  Please reference {@link PointerCreateForm} for more
     *  information
     */
    function form_init_elements()
    {
        parent::form_init_elements();
        
        $enabled = new FEListBox(_("Status"));

        $enabled->set_list_data(array('Enabled'=>1,'Disabled'=>'0'));

        $this->add_element($enabled);
    }

//}}}
//{{{   form_init_data()

    /** 
     *  This method initializes the form to the values that are in the database
     */
    function form_init_data()
    {
        global $conf;
        
        $pntr = new Pointer($_REQUEST['id']);
        $formary = $this->remap($pntr->toArray(), FORM);
        
        foreach($formary as $label=>$val)
            $this->set_element_value($label, $val);

        $this->set_element_value(_("Time to Live"), $conf['soa']['default_pointer_ttl']);
    }

//}}}
//{{{   form_action()

    /**
     *  This method updates the pointer in the database if the form is valid
     *  see {@link PointerCreateForm} for validation information.
     *
     *  @return Boolean True if the action was performed successfully
     */
    function form_action()
    {
        global $user;
        $res = true;
        
        switch(strtolower($this->get_action())) {
    
        case 'submit':
            $array = $this->get_all_element_values();
            $pntr = new Pointer($_REQUEST['id']);
            $pntr->query("BEGIN");
            $pntr->setFrom($this->remap($array, DBDO));

            if($pntr->update() === false) {
                $pntr->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), 
                                _("Could not update Pointer entry in Database."));
            } else {
                $pntr->query("COMMIT");
                $pntr->write_history(_("Pointer Updated"), ACTION_MODIFY, $user->username,
                                    $_SESSION['zone']);
            }
            break;

        default:
            die_hard(_("Invalid form action"));
            break;
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  Display a success message if the action worked
     *
     *  @return DIVTag  Div with a success message
     */
    function form_success()
    {
        $div = html_div('msuccess');
        $div->add(_("Successfully Modified Pointer"), html_br());
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $sublink = html_a(web_root()."pointer.php", _("[Return to Pointer List]"));

        $div->add($indlink, $sublink);

        return $div;
    }

//}}}
//{{{   form_hidden()

    /**
     *  Add some hidden values that we'll need later.  The ID of the CNAME and the
     *  get action so we can get back to the form.
     *
     *  @return Container   Container that has the hidden elements in it
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));
    }

//}}}

}

?>
