<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2006 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
 *  SOAEditForm.php - contains the {@link SOAEditForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/SOACreateForm.php");

class SOAEditForm extends SOACreateForm
{

//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that will be used.  Please refer
     *  to {@link SOACreateForm} for more information.
     */
    function form_init_elements()
    {
        parent::form_init_elements();
        $comment = new FETextArea(_("Comment"), true, 5, 40);

        $this->add_elements($comment);
    }

//}}}
//{{{   form_init_data()
   
    /**
     *  This method sets the initial values for the form elements based off of
     *  which record we're updating.
     */
    function form_init_data()
    {
        $soa = new SOA($_REQUEST['id']);
        $formarray = $this->remap($soa->toArray(), FORM);
        
        foreach($formarray as $label=>$val)
            $this->set_element_value($label, $val);
        
        $this->set_element_value(_("Comment"), _("Modified SOA"));
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method establishes the hidden elements for the form.
     *
     *  @return Container   Container with the hidden elements for the form
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));                                
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what actually handles the updating of the record using
     *  the form values.
     *
     *  @return Boolean True if action succeeds
     */
    function form_action()
    {
        global $user;
        
        $res = true;
        
        switch(strtolower($this->get_action())) {

        case 'submit':
            $soa = new SOA($_REQUEST['id']);
            $soa->query("BEGIN");
            $ary = $this->get_all_element_values();

            $soa->setFrom($this->remap($ary, DBDO));
            $soa->delegated = ($ary[_('Delegated')] == 1 ? 1 : 0);

            if(!$soa->update()) {
                $soa->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), _("Failed to modify SOA Record in Database"));
            } else {
                $soa->query("COMMIT");
                $soa->write_history($ary[_('Comment')], ACTION_MODIFY, $user->username);
            }
            break;

        default:
            die_hard(_("Invalid form action"));
            break;

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This displays a happy message to the user letting them know that the update
     *  worked.
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success() 
    {
        $div = html_div('msuccess');
        $div->add(_("Successfully Modified SOA Record"), html_br());
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $sublink = html_a(web_root()."soa.php", _("[Return to SOA List]"));

        $div->add($indlink, $sublink);

        return $div;
    }

//}}}

}

?>
