<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2006 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Installation procedures for Maintain.
  *
  * This htdoc operates with one REQUEST variable: state.
  * 'state' is actually the ORing of all of the stages
  * of the install process, but this script will execute
  * the steps in the proper chronological order.
  *
  * @package    install
  * @subpackage htdocs
  */
require_once("includes.php");
require_once(INCLUDEDIR . "/install.php");
require_once(FORMDIR."/InstallForms.php");

global $conf;

// if there is no reason for us to install or upgrade, get out of here.
if (install_status() === true)
    header("Location: index.php");

// setup the HTML container (phphtmllib)
$content = container();

// set the page title
$title = html_h1(_("Maintain").' '.VERSION.' '._("Installer/Updater"));
$content->add($title);

// load the proper skin
$skin = Skin::Factory("maintain");
$state = @$_REQUEST['state'];   // no error output, since request may be empty
$first_visit = $state == null; //@$_REQUEST['first_visit'];

//to add the "Click here to continue" link or not
$add_next = false;

// if this is the first visit here, let the reader know what's going on
if ($first_visit) {
    $maintain_root = $conf['paths']['prefix'];
    $welcome_header = _("Welcome to Maintain!");
    $welcome_body = container(_("The next few screens will help you get Maintain installed or updated and configured."), html_br(2), _("The installer will attempt to detect an existing database or install a new one otherwise."), html_br(), _("Please make sure that the correct database information is in:"), html_br(2), "{$maintain_root}/maintain.cfg");
    $body = container($welcome_header, $welcome_body);
    $add_next = true;
} else if ($state & INSTALL_DB)    {
    $header = _("Initialize the database");
    $idb = new FormProcessor(new InstallDBForm, "installdb");
    $idb->set_render_form_after_success(false);
    $body = container();
    $body->add($idb);
    if ($idb->is_action_successfull())
        $add_next = true;

} else if ($state & UPDATE_DB)  {
    $dbdo = DB_DataObject::Factory('update_info');
    $dbdo->find(true);
    $res = $dbdo->value;
    if(isset($user)) {
        $res1 = $user->is_admin();
    }
    else {
        $res1 = false;
    }
    if($res > 240001 && !$res1) {
        header("Location: unavailable.php");
        exit;
    }
    $header = _("Update Database");
    $body = container();
    
    $ver = new DAO_Update_Info;
    $ver->query("SELECT * FROM update_info WHERE `key` = 'db_version';");
    $ver->fetch();
    $version = $ver->value;

    if ($version > MAINTAIN_DB_VERSION) {
        $div = container();
        $div->add(sprintf(_("Maintain has detected that your database is newer (%s) than the version of Maintain you are running (%s)."), $version, MAINTAIN_DB_VERSION));
        $div->add(html_br(1), _("You have to upgrade your Maintain instance for to work with this database. Until then you can't work with Maintain, sorry."));
        $div->add(html_br(2));
        $body->add($div);
    } else {
        $uf = new FormProcessor(new UpdateDBForm, "updateform");
        $uf->set_render_form_after_success(false);
        $body->add(html_br(2), $uf);
        if ($uf->is_action_successfull())
            $add_next = true;
    }
    
} else if ($state & RENAME_LINKS_INI) {
    /* rename the <dbname>.links.ini file to match $database name */
    $header = _("Rename &lt;dbname&gt;.links.ini");

    require_once(CLASSDIR . "/DAOWriter.php");
    $writer = new DAOWriter();
    $body = container();

    $result = $writer->verifyLinksIni(false);
    if ($result !== true) {
        $body->add($writer->getFormattedMessages('html'));
        $cmd_string = container(html_strong(_("Please execute the command:")), html_br(2), html_strong($result));
        $body->add($cmd_string);
    } else {
        $body->add(_("Links.ini file is properly named."));
        $add_next = true;
    }

} else if ($state & INSTALL_ADMIN_PASSWORD) {

    /* Set the admin password */
    $header = _("Set a password for the Admin account");

    $body = new FormProcessor(new AdminPasswordForm(), "adminpass");
    $body->set_render_form_after_success(false);
    if ($body->is_action_successfull())
        $add_next = true;

} else if ($state & INSTALL_INITIAL_ZONE) {

    $header = _("Create an initial zone.");

    $body = new FormProcessor(new InitialZoneCreateForm(), "initial_zone");
    $body->set_render_form_after_success(false);
    if ($body->is_action_successfull())
        $add_next = true;

} else {
    /* this case when everything is set up and there's nothing for us to do */

    $header = _("Thanks for installing Maintain!");
    $body = _("Click continue now to start using Maintain.");
    $add_next = true;
}

//If we're not finished installing figure out where we are and forward there
$newstate = install_status();
if ($newstate === true)
    $next = html_a('index.php', _("Click here to continue"));
else
    $next = html_a('install.php?state='.$newstate, _("Click here to continue"));


// add the content to the page's container
$content->add(html_h3($header),$body,html_hr(), ($add_next ? $next : null));

echo $skin->displayInstall($content);

?>
