<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC3/htdocs/zone.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Michael Clay <claym@osuosl.org>                               |
// |          Zack Bartel  <bartel@osuosl.org>                              |
// +------------------------------------------------------------------------+
// $Id: zone.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * Zone Pages
  *
  * @package    htdocs
  */

require_once("includes.php");
require_once(HTCLASS."/datalist/ZoneDataList.php");
require_once(HTCLASS."/datalist/HistoryDataList.php");
require_once(HTCLASS."/datalist/RangeZoneDataList.php");
require_once(HTCLASS."/datalist/WorkgroupZoneDataList.php");
require_once(FORMDIR."/ZoneCreateForm.php");
require_once(FORMDIR."/ZonePHostsConfirm.php");
require_once(FORMDIR."/ZonePurgeConfirm.php");
require_once(FORMDIR."/ZoneEditForm.php");
require_once(FORMDIR."/ZoneDomainForm.php");
require_once(FORMDIR."/MaintainDeleteConfirm.php");
require_once(FORMDIR."/ZoneUserAddForm.php");
require_once(FORMDIR."/RangeZoneBulkDeleteConfirm.php");
require_once(FORMDIR."/ZoneWorkgroupAddForm.php");
require_once(FORMDIR."/ZoneMergeForm.php");
require_once(FORMDIR."/ZoneWorkgroupBulkDeleteConfirm.php");


session_start();

setup_user_environment();

$title = html_span("title");

$skin = Skin::Factory($user->color_template);

$content = container($title);

switch(strtolower($_REQUEST['action'])) {

case 'purge_zone':
    if(!$user->is_zone_admin($zone))
        header("Location: denied.php");
    $title->add(_("Purge Zone"));
    $content->add($title);
    
    $zpc = new FormProcessor(new ZonePurgeConfirm, 'purgezone');
    $zpc->set_render_form_after_success(false);
    $content->add($zpc);
    break;

case 'purge':
    if(!$user->is_zone_admin($zone))
        header("Location: denied.php");
    $title->add(_("Purge Hosts"));
    $content->add($title);

    $zphf = new FormProcessor(new ZonePHostsConfirm,"purgehosts");
    $zphf->set_render_form_after_success(false);
    $content->add($zphf);
    break;
case 'delete_ranges':
case 'delete ranges':
    if (!$user->is_admin())
        header("Location: ".web_root()."denied.php");
    
    if (isset($_REQUEST['zrmaps']))
        $zrids = $_REQUEST['zrmaps'];
    else
        $zrids = array($_REQUEST['zrid']);

    $rz = new FormProcessor(new RangeZoneBulkDeleteConfirm($zrids), "zonerangedel");
    $rz->set_render_form_after_success(false);
    $content->add($rz);
    break;

case 'merge':
    if (!$user->is_admin())
        header("Location: denied.php");
    
    $title->add("Merging Zones");
    $content->add($title, html_br(2));
    $zmform = new FormProcessor(new ZoneMergeForm, "zonemerge");
    $zmform->set_render_form_after_success(false);
    $content->add($zmform);
    
    break;

    
case 'create':
    if(!$user->is_admin())
        header("Location: denied.php");
    $title->add(_("Create Zone"));
    $content->add($title);
    
    $zcf = new FormProcessor(new ZoneCreateForm,"zonecreate");
    $zcf->set_render_form_after_success(false);
    $content->add($zcf);
    break;

case _('remove workgroups'):
case 'remove_workgroups':
case _('remove_from_zone'):
    if (!$user->is_admin())
        header("Location: denied.php");

    //did we get here from the Remove button or the action button in the list
    if (isset($_REQUEST['zwmaps']))
        $zwids = $_REQUEST['zwmaps'];
    else
        $zwids = array($_REQUEST['zwid']);

    $zwdel = new FormProcessor(new ZoneWorkgroupBulkDeleteForm($zwids), "zwdel");
    $zwdel->set_render_form_after_success(false);
    $content->add($zwdel);
    break;

/* Shows history for all zones */
case 'show_all_history':
        $title->add(_("Viewing Zone History List"));
        $content->add($title);
        $dl = new HistoryDataList(_("Zone History List"));
        $dl->user_setup();
        $content->add($dl);
break;
/* Shows history for this zone */
case 'show_history':
    if(!$user->is_guest($zone))
        header("Location: denied.php");
    if(isset($_REQUEST['id'])) {
        $title->add(_("Viewing Zone History"));
        $content->add($title);
        $dbobj = new Object_History($_REQUEST['id']);
        $dbobj->id = $_REQUEST['id'];
        $dbobj->find(1);
        $content = Module::apply_filter("object_info",$content,$dbobj);
    } else {
        $title->add(_("Viewing Zone History List"));
        $content->add($title);
        $dl = new ZoneHistoryDataList(_("Zone History List"));
        $dl->user_setup();
        $content->add($dl);
    }
    break;
case 'zone_range_endis':
    if (!$user->is_admin())
        header("Location: denied.php");

    toggleEnabled($_REQUEST['zrid']);
    header("Location: {$_SERVER['SCRIPT_NAME']}?action=edit&id={$_REQUEST['id']}");
case _('toggle default'):
    
    if (!$user->is_zone_admin())
        header("Location: denied.php");
    $zone_wk_ids = $_REQUEST['zwmaps'];

    //If the user selected more than one workgroup to "toggle default"
    //Then we are just going to take the last one, make it default
    //And set the rest to not default 
    if(is_array($zone_wk_ids)){
        $zw_id = array_shift($zone_wk_ids);
        $dbdo = DB_DataObject::Factory('zone_workgroup');
        $dbdo->is_default = 1;
        $dbdo->whereAdd("id='{$zw_id}'");
        $dbdo->update(DB_DATAOBJECT_WHEREADD_ONLY);
        //kill the rest
        $dbdo = DB_DataObject::Factory('zone_workgroup');
        $dbdo->is_default = 0;
        $dbdo->whereAdd("id!='{$zw_id}'");
        $dbdo->update(DB_DATAOBJECT_WHEREADD_ONLY);
     }
case 'edit':
    if (!$user->is_zone_admin())
        header("Location: denied.php");
    $title->add(_("Editing Zone"));
    $content->add($title);

    if(!$user->is_zone_admin($zone))
        header("Location: ".web_root()."denied.php");

    $zoneedit = new FormProcessor(new ZoneEditForm, "zedit");
    $zoneedit->set_render_form_after_success(false);
    $content->add($zoneedit);
    
    $content->add(html_br());
    $title = html_span("title");
    $title->add(_("Domain to Zone mappings"));
    $content->add($title);
    $zonedom = new FormProcessor(new ZoneDomainForm, "zdom");
    $content->add($zonedom,html_br());

    $title = html_span("title");
    $title->add(_("Update User permissions on this Zone"));
    $content->add($title);
    $zoneusr = new FormProcessor(new ZoneUserAddForm, "zusr");
    $content->add($zoneusr, html_br());

    $title = html_span("title");
    $title->add(_("Zone to Range Mapping"));
    $content->add($title);
    $zonerange = new RangeZoneDataList(_("Zone to Range Mappings"));
    $zonerange->user_setup();
    $content->add($zonerange, html_br());

    $title = html_span("title");
    $title->add(_("Workgroups in this Zone"));
    $content->add($title);
    $zw = new WorkgroupZoneDataList(_("Zone to Workgroup Mapping"));
    $zw->user_setup();
    $content->add($zw, html_br());

    $title = html_span("title");
    $title->add(_("Add a Workgroup to this Zone"));
    $content->add($title);
    $zaw = new FormProcessor(new ZoneWorkgroupAddForm, 'zwadd');
    $content->add($zaw);
    

    break;
    
case 'delconfirm':
    $title->add("Deleting Zone");

    if (!$user->is_zone_admin($zone))
        header("Location: denied.php");

    $del = new FormProcessor(new MaintainDeleteConfirm("zone", "name"), "zdel");
    $del->set_render_form_after_success(false);
    $content->add($del);
        
    break;

case 'show':

default:
    $title->add(_("Listing Zones"));
    $content->add($title);

    $zl = new ZoneDataList("Zones");
    $zl->user_setup();
    $content->add($zl);

}

function toggleEnabled($id)
{
    $obj = DB_DataObject::Factory('zone_range');
    $obj->get($id);
    $obj->enabled = !$obj->enabled;
    $obj->update();
}



print $skin->display($content);

?>
