<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Authentication methods
  *
  * @author $Author$
  * @filesource
  * @package    includes
  */

//{{{ authenticate()

/**
 * This function is responisble for trying to authenticate the user.
 * 
 * @global  array   Maintain global config array
 * @param   string  the user's name
 * @param   string  the user's password
 * @return  mixed   true on successful login, error string otherwise
 */
function authenticate($username, $password)
{
    global $conf;
    
    // determine how the user can login
    $user = DB_DataObject::Factory('user');
    $user->selectAdd();
    $user->selectAdd('type');

    // if the user wasn't found, get out of here
    if (!$user->get($username)) {
        //return $conf['auth']['user_not_found'];
        // changed to bad password for not giving the "bad guys"
        // too much information
        return $conf['auth']['login_failed'];
    }
 
    // now, make sure the user's auth method is enabled
    // TODO not 100% sure which error message to use
    if (!is_auth_enabled($user->type)) {
        return $conf['auth']['bad_auth_cred'];
    }

    // perform the authentication
    $mod = new $user->type;
    $result = $mod->auth($username, $password);
    if ($result !== true) {
        return $mod->getStatus();
    }

    // or return false if we can't find the module
    return $result;

}
// }}}
//{{{ logout()
/**
 * This function is meant to logout the current session and put them 
 * back at the login page.
 *
 * @global mixed $conf Maintain global config array
 * @return void
 */
function logout()
{
    global $conf;
    
    sess_destroy();
    $login_page = web_root() . 'login.php' ;
    header("Location: $login_page");
    exit();
} //END logout()
//}}}
//{{{ sess_destroy()

/**
 * This function destroys the session.
 *
 * @global  mixed   Maintain global config array
 * @param   int     the session id number (optional)
 * @return  void
 */
function sess_destroy($id=0)
{
    global $conf;

    // if an id is not passed in, figure it out
    if($id == 0) {
        session_start();
    }
    session_destroy();
} //END sess_destroy();

//}}}
// {{{ valid_auth_types()

/**
  * Obtain a list of enabled authentication types
  *
  * @return array   enabled authentication methods by name
  */
function valid_auth_types()
{
    global $conf;

    $types = array();
    $auths = Module::do_action('auth_modules');
    foreach ($auths as $a)
        $types[] = $a['result'];
    // return the list
    return $types;
}

// }}}
// {{{ is_auth_enabled()

/**
  * Check if a particular authentication method is available
  *
  * This is MUCH faster than calling valid_auth_types and iterating through.
  *
  * @param  string  authentication method ('maintain' or the name of an
  *                 authenticating module)
  * @return boolean true if given auth type is enabled, false otherwise
  */
function is_auth_enabled($type)
{
    // the native authentication method is always enabled
    if ($type == DEFAULT_AUTH) {
        return true;
    }

    // see if the given type is even an enabled modules
    $module = DB_DataObject::Factory('modules');
    $module->name = $type;
    $module->enabled = 1;
    if (!$module->find(true)) {
        return false;
    }

    // make sure this module is really an authenticating module
    $mod = new $module->name;
    if (!is_a($mod, 'AuthModule')) {
        return false;
    }

    return true;
}

// }}}

?>
