<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: maintain_nav.php 4822 2006-09-15 18:57:40Z morgabra $

/**
  * maintain_nav.php - contains the {@link Maintain_Nav} class
  *
  * @author $Author: $
  * @filesource
  * @package    modules
  * @subpackage maintain_nav
  */

require_once MODDIR.'/core_module.php';

/**
  * This is the 'Maintain_Nav' module for Maintain, providing a basic nav bar
  *
  * @package    modules
  * @subpackage maintain_nav
  */
class Maintain_Nav extends Core_Module {

//{{{ vars

//}}}

/* Overridden Module Methods */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {

        return true;

    }

// }}}
// {{{ register_hooks() (register methods to Maintain module hooks)

/**
  * Connect this module to the module hooks it should reply to
  *
  * @access public
  * @return void
  */
    function register_hooks()
    {
        Module::add_filter('navlinks', 'makeHomeLink', 0); // priority 0: always on top
        Module::add_filter('navlinks', 'makeGeneralNavLinks', 15);
        Module::add_filter('navlinks', 'makeLogoutLink', 20, NO_ACCESS); // priority 20: logout always at the bottom
    }
// }}}

/* This module's methods */
//{{{ makeHomeLink()
    /**
      * add maintain navigation links to menu object
      *
      * @global array site-wide image locations
      * @global array user object to determine admin status
      * @param object menu object
      * @return object menu object with added nav links
      */
    function makeHomeLink($menu)
    {
        global $images, $user;

        $home_img = html_img($images[IMG_HOME]);
        
        $link = html_a(web_root()."index.php",container($home_img,_("Home")));
        $menu->add($link);

        return $menu;
    }

// }}}
//{{{ makeGeneralNavLinks() (was: getNavLinks)
    /**
      * add maintain navigation links to menu object
      *
      * @global array site-wide image locations
      * @global array user object to determine admin status
      * @param object menu object
      * @return object menu object with added nav links
      */
    function makeGeneralNavLinks($menu)
    {
        global $images, $user;

        $links = array();

        $pref_img = html_img($images[IMG_PREF]);
        $admin_img = html_img($images[IMG_ADMIN]);
        $help_img = html_img($images[IMG_HELP]);
        
        $links['prefs'] = html_a(web_root()."preferences.php", container($pref_img, _("Preferences")));
        $links['help'] = html_a(web_root()."help.php?action=show", container($help_img, _("Help")));
        
        if(isset($user) && $user->is_admin())
            $links['admin'] = html_a(web_root()."admin.php",container($admin_img,_("Admin")));

        foreach ($links as $link) {
            $menu->add($link);
        }

        return $menu;
    }

// }}}
// {{{ makeLogoutLink()
    /**
      * adds logout nav link to the bottom
      *
      * @global array site-wide image locations
      * @param object menu object
      * @return object modified menu object
      */
    function makeLogoutLink($menu)
    {
        global $images;

        $logout_img = html_img($images[IMG_LOGOUT]);
        $lo_link = html_a(web_root()."logout.php",container($logout_img,_("Logout")));

        $menu->add($lo_link);

        return $menu;
    }
// }}}
}

?>
